from flask import Blueprint, request, jsonify
from werkzeug.exceptions import NotFound, BadRequest, Forbidden

from flaschengeist import logger
from flaschengeist.plugins import Plugin
from flaschengeist.decorator import login_required
from flaschengeist.controller import userController

users_bp = Blueprint("users", __name__)
_permission_edit = "users_edit_other"
_permission_set_roles = "users_set_roles"
_permission_delete = "users_delete_other"


class UsersPlugin(Plugin):
    def __init__(self, config):
        super().__init__(blueprint=users_bp, permissions=[_permission_edit, _permission_delete, _permission_set_roles])

    #################################################
    #                     Routes                    #
    #                                               #
    # /users          POST: register new            #
    #                  GET: get all users           #
    # /users/<uid>     GET: get user with uid       #
    #                  PUT: modify user             #
    #               DELETE: remove user             #
    #################################################


@users_bp.route("/users", methods=["POST"])
def __registration(self):
    logger.debug("Register new User...")
    return jsonify({"ok": "ok... well not implemented"})


@users_bp.route("/users", methods=["GET"])
@login_required()
def __list_users(**kwargs):
    logger.debug("Retrieve list of all users")
    users = userController.get_users()
    return jsonify(users)


@users_bp.route("/users/<uid>", methods=["GET"])
@login_required()
def __get_user(uid, **kwargs):
    logger.debug("Get information of user {{ {} }}".format(uid))
    user = userController.get_user(uid)
    if user:
        return jsonify(user)
    raise NotFound


@users_bp.route("/users/<uid>", methods=["DELETE"])
@login_required(permission=_permission_delete)
def __delete_user(uid, **kwargs):
    logger.debug("Delete user {{ {} }}".format(uid))
    user = userController.get_user(uid)
    userController.delete(user)
    return jsonify({"ok": "ok"})


@users_bp.route("/users/<uid>", methods=["PUT"])
@login_required()
def __edit_user(uid, current_session, **kwargs):
    logger.debug("Modify information of user {{ {} }}".format(uid))
    user = userController.get_user(uid)
    data = request.get_json()

    password = None
    new_password = data["new_password"] if "new_password" in data else None

    if uid != current_session._user.userid:
        if not user.has_permission(_permission_edit):
            return Forbidden
    else:
        if "password" not in data:
            raise BadRequest("Password is missing")
        password = data["password"]

    for key in ["firstname", "lastname", "display_name", "mail"]:
        if key in data:
            setattr(user, key, data[key])

    if "roles" in data:
        if not user.has_permission(_permission_set_roles):
            raise Forbidden
        userController.set_roles(user, data["roles"])

    userController.modify_user(user, password, new_password)
    userController.update_user(user)
    return jsonify({"ok": "ok"})
